/*
Package log provides Logger
*/
package log

import (
	"context"
	"log"

	"github.com/vardius/golog"

	"github.com/vardius/go-api-boilerplate/pkg/metadata"
)

const (
	// Prefix text to prefix to each log entry generated by the Logger
	TraceIDPrefix = "TraceID: %s "
)

// Logger allow to create logger based on env setting
type Logger struct {
	golog.Logger
}

// New creates new logger based on environment
func New(env string) *Logger {
	l := golog.New()
	if env != "development" {
		l.SetVerbosity(golog.DefaultVerbosity &^ golog.Debug)
	}

	l.SetFlags(log.Ldate | log.Ltime | log.Lmicroseconds | log.LUTC)

	return &Logger{l}
}

func (l *Logger) Debug(ctx context.Context, format string, args ...interface{}) {
	mtd, ok := metadata.FromContext(ctx)
	if ok {
		args = append([]interface{}{mtd.TraceID}, args...)
		format = TraceIDPrefix + format
	}

	l.Logger.Debug(ctx, format, args...)
}

func (l *Logger) Info(ctx context.Context, format string, args ...interface{}) {
	mtd, ok := metadata.FromContext(ctx)
	if ok {
		args = append([]interface{}{mtd.TraceID}, args...)
		format = TraceIDPrefix + format
	}

	l.Logger.Info(ctx, format, args...)
}

func (l *Logger) Warning(ctx context.Context, format string, args ...interface{}) {
	mtd, ok := metadata.FromContext(ctx)
	if ok {
		args = append([]interface{}{mtd.TraceID}, args...)
		format = TraceIDPrefix + format
	}

	l.Logger.Warning(ctx, format, args...)
}

func (l *Logger) Error(ctx context.Context, format string, args ...interface{}) {
	mtd, ok := metadata.FromContext(ctx)
	if ok {
		args = append([]interface{}{mtd.TraceID}, args...)
		format = TraceIDPrefix + format
	}

	l.Logger.Error(ctx, format, args...)
}

func (l *Logger) Critical(ctx context.Context, format string, args ...interface{}) {
	mtd, ok := metadata.FromContext(ctx)
	if ok {
		args = append([]interface{}{mtd.TraceID}, args...)
		format = TraceIDPrefix + format
	}

	l.Logger.Critical(ctx, format, args...)
}
